#ifndef XSHOSTAKOVICH_HH
#define XSHOSTAKOVICH_HH

/* xshostakovich.hh
   Copyright (C) 1997-8 Eddie Kohler, eddietwo@lcs.mit.edu
   This file is part of PORTABLE XSHOSTAKOVICH++ 98.

   PORTABLE XSHOSTAKOVICH++ 98 is free software. It is distributed under the
   GNU Public License, version 2 or later; you can copy, distribute, or alter
   it at will, as long as this notice is kept intact, this source code is made
   available, and you are crazy nuts. Warranty? Ha! */

#include <X11/Xlib.h>
#include "gif.h"
#include "gifx.h"

class GifPicture {

  Gif_XContext *_gfx;
  Gif_Stream *_gfs;
  Pixmap _pixmap;

  void create();
  
 public:

  GifPicture(Gif_Record *, Gif_XContext *);

  unsigned width() const	{ return _gfs->images[0]->width; }
  unsigned height() const	{ return _gfs->images[0]->height; }
  Pixmap pixmap()		{ if (!_pixmap) create(); return _pixmap; }
  
};


class Shostakovich {

  Display *_display;
  Window _window;
  Colormap _colormap;
  Gif_XContext *_gfx;

  unsigned _width;
  unsigned _height;
  int _origin_x;
  int _origin_y;

  const char *_saying;
  int _saying_width;
  bool _quiet;

  XColor _background;
  bool _background_mutable;

  GC _gc;
  GC _eye_gc;
  
  bool _tongueing;
  bool _left_eye_dead;
  bool _right_eye_dead;
  int _just_iconified;

  GifPicture *_dsch;
  GifPicture *_smile;
  GifPicture *_tongue;
  GifPicture *_left_eye;
  GifPicture *_right_eye;
  GifPicture *_left_eyeball;
  GifPicture *_right_eyeball;
  
  // The color of Shostakovich's background:
  static const char * const BackgroundColor = "#497955";
  
  // The upper-left corner of the smile pixmap, in relation to Shostakovich.
  static const int SmileOffsetX = 268;
  static const int SmileOffsetY = 327;
    
  // The upper-left corner of the tongue pixmap, in relation to Shostakovich.
  static const int TongueOffsetX = 269;
  static const int TongueOffsetY = 329;
    
  // The upper-left corner of the X'ed out right eye, in relation to
  // Shostakovich. Also, the eye's width and height, for offset calculation
  // for the mouse-tracking eye-following stuff.
  static const int RightEyeOffsetX = 225;
  static const int RightEyeOffsetY = 215;
  static const int RightEyeWidth = 60;
  static const int RightEyeHeight = 78;
    
  // The upper-left corner of the X'ed out left eye, in relation to
  // Shostakovich. Also, the eye's width and height, for offset calculation
  // for the mouse-tracking eye-following stuff.
  static const int LeftEyeOffsetX = 327;
  static const int LeftEyeOffsetY = 211;
  static const int LeftEyeWidth = 53;
  static const int LeftEyeHeight = 65;
    
  // The upper-left corner of the eye mask bitmap, which is used to mask out
  // everything in the picture except the eyes, for the eye-following stuff.
  static const int EyesMaskOffsetX = 257;
  static const int EyesMaskOffsetY = 227;
    
  // The upper-left corner of the right eyeball pixmap, in relation to
  // Shostakovich.
  static const int RightEyeballOffsetX = 246;
  static const int RightEyeballOffsetY = 225;
    
  // The upper-left corner of the left eyeball pixmap, in relation to
  // Shostakovich.
  static const int LeftEyeballOffsetX = 335;
  static const int LeftEyeballOffsetY = 220;

  void draw_part(GifPicture *, bool, int, int);
  void draw_smile(bool);
  void draw_tongue(bool);
  void draw_left_eye(bool);
  void draw_right_eye(bool);
  void draw_string(int dx, int dy, const char *);

  void follow_left_eye(const XMotionEvent &);
  void follow_right_eye(const XMotionEvent &);

  void die();
  void do_drag();
  
  bool in_dsch_rect(int px, int py, int x, int y, int w, int h) const;
  
 public:

  static const char *saying;
  static const char *font_name;
  static const char *saying_color_name;
  static bool quiet;
  
  Shostakovich(Display *, Window, Colormap);
  
  void draw();
  bool handle_event(XEvent &);
  
};

#endif
